@@ -68,8 +68,9 @@ proc ical_with_item {v} {
 proc ical_with_mod_item {v} {
     if ![ical_with_item x] {return 0}
 
-    if [cal readonly [$x calendar]] {
-	ical_error "[$x calendar]: permission denied"
+    if [catch {set cal [$x calendar]}] {return 0}
+    if [cal readonly $cal] {
+	ical_error "$cal: permission denied"
 	return 0
     }
 
@@ -161,7 +162,8 @@ action ical_close always {Close ical window} {} {
 action ical_cut_or_hide item {Delete selected item} {} {
     if ![ical_with_item i] return
 
-    if {[$i owned] || ([$i calendar] == [cal main])} {
+    if [catch {set cal [$i calendar]}] return
+    if {[$i owned] || ($cal == [cal main])} {
 	ical_cut
     } else {
 	ical_hide
@@ -177,6 +179,7 @@ action ical_cut witem {Delete selected item even if owned by another user} {} {
 
 action ical_hide item {Hide selected item} {} {
     if ![ical_with_item i] return
+    if [catch {set cal [$i calendar]}] return
 
     # Since the hide entry will go in the main calendar,
     # check for permission there.
@@ -189,7 +192,7 @@ action ical_hide item {Hide selected item} {} {
     # unless we have to split a repeating item.
 
     if [$i repeats] {
-	if [cal readonly [$i calendar]] {
+	if [cal readonly $cal] {
 	    # Tell user that the item cannot be split.
 	    # See if all entries should be hidden.
 	    if {![yes_or_no [ical_leader] [join {
@@ -246,6 +249,7 @@ action ical_import writable {Parse X selection as an item and add it to calendar
 
 action ical_makeunique witem {Create unique occurrence of selected item} {} {
     if ![ical_with_mod_item i] return
+    if [catch {set cal [$i calendar]}] return
     if ![$i repeats] {
 	ical_error "Item does not repeat"
 	return
@@ -254,7 +258,7 @@ action ical_makeunique witem {Create unique occurrence of selected item} {} {
     set c [$i clone]
     $c date [ical_date]
     $i deleteon [ical_date]
-    cal add $c [$i calendar]
+    cal add $c $cal
 }
 
 action ical_moveitem witem {Move item to another calendar} {calendar} {
@@ -273,8 +277,16 @@ action ical_addinclude writable {Include calendar} {} {
 	return
     }
 
+    # Find last include file name and use as initial value for dialog
+    global ical
+    if ![info exists ical(last_include)] {
+	set last [file dirname $ical(calendar)]
+    } else {
+	set last $ical(last_include)
+    }
     if ![get_file_name [ical_leader] "Include Calendar"\
-	     "Select calendar file to include." filename] return
+	     "Select calendar file to include." filename $last] return
+    set ical(last_include) [file dirname $filename]
 
     # Some sanity checking
     if [catch {set e [file exists $filename]} msg] {
@@ -294,7 +306,7 @@ action ical_addinclude writable {Include calendar} {} {
 	}
     }
 
-    if [catch {cal include $filename} error] {
+    if [catch {cal include [ical_expand_file_name $filename]} error] {
 	ical_error $error
     }
 }
@@ -443,7 +455,7 @@ action ical_link_to_file witem {Create a link to a file} {} {
 	     "Select file to which link should be created."\
 	     filename $initial] return
 
-    $i option Link file://localhost$filename
+    $i option Link file://localhost[ical_expand_file_name $filename]
 }
 
 action ical_link_to_uri witem {Create a link to a Web document} {} {
@@ -524,6 +536,8 @@ action ical_toggle_todo witem {Make item a todo item} {} {
 
 action ical_toggle_done witem {Mark todo item as done} {} {
     if ![ical_with_mod_item i] return
+    if [catch {set cal [$i calendar]}] return
+
     if ![$i todo] {
 	ical_error "Item is not a todo item"
 	return
@@ -541,7 +555,7 @@ action ical_toggle_done witem {Mark todo item as done} {} {
 	# XXX Make clone that does not occur on or before the current date
 	set c [$i clone]
 	$c start [expr $date+1]
-	cal add $c [$i calendar]
+	cal add $c $cal
     }
 
     # Modify this occurrence
@@ -664,33 +678,29 @@ action ical_defalarms writable {Change default alarm settings} {} {
     alarmer recompute
 }
 
-action ical_cmdkey always {Change command key bindings} {} {
-    global keymap
-
-    if [edit_key_binding keymap(command)] {
-	# Redo the binding map and set-up menu accelerators
-	ical_make_bindings IcalCommand $keymap(command)
-	trigger fire keybind
+action ical_edit_key always {Change command key bindings} {} {
+    if [cal readonly] {return}
+    if [define_key [ical_leader] key] {
+	ical_define_key $key
     }
 }
 
-proc edit_key_binding {var} {
-    upvar $var map
-    if ![define_key [ical_leader] key] {return 0}
-
-    # Remove duplicates from map
-    lappend map $key
-    foreach x $map {set val([lindex $x 0]) [lindex $x 1]}
-
-    # Create new map
+proc ical_define_key {key} {
+    # We want the map to be sorted and without duplicates
+    catch {foreach {s c} [cal option Keybindings] {set val($s) $c}}
+    set val([lindex $key 0]) [lindex $key 1]
     set map {}
     foreach key [lsort [array names val]] {
 	if [string compare $val($key) ""] {
-	    lappend map [list $key $val($key)]
+	    lappend map $key $val($key)
 	}
     }
 
-    return 1
+    if [catch {cal option Keybindings $map} msg] {
+	error_notify [ical_leader] $msg
+    } else {
+	trigger fire keybind
+    }
 }
 
 action ical_gripe always {Gripe to the author of ical} {} {
@@ -703,6 +713,11 @@ action ical_help always {Display help on ical} {} {
     Ical_Doc $ical(doc)
 }
 
+action ical_tcl_interface always {Display the Tcl interface to ical} {} {
+    global ical
+    Ical_Doc $ical(tcl_interface_doc)
+}
+
 action ical_about always {Display information about this version of ical} {} {
     show_about [ical_leader]
 }
