@@ -5,26 +5,60 @@
 #
 # Rep
 #	alarms		list of alarm times in minutes
-#	pending		list of pending alarms; each element is list --
-#				{<item> <fire time> <appt start time>}
+#	pending		list of pending alarms;
+#			    [<item> <fire time> <appt start time>]*
+#	shutlist	list of item uids for which we said "shutup" in
+#			the last hour
 
 class Alarmer {} {
     set slot(alarms)	[list 0 5 10 15]
     set slot(pending)	""
-
-    # Consider more alarms once every six hours
-    thread_periodic [expr 6*60*60*1000] [list $self recompute]
-
-    # Check new alarms on minute boundaries
-    thread_minute [list $self fire]
+    set slot(shutlist)	""
 
     trigger on add	[list $self add]
     trigger on delete	[list $self remove]
     trigger on change	[list $self change]
     trigger on flush	[list $self recompute]
 
-    # Initial recomputation
+    # Start various threads
+    $self recompute_thread
+    $self fire_thread
+    $self clear_shutup_thread
+}
+
+# Runs "Alarmer::recompute" once every six hours
+method Alarmer recompute_thread {} {
     $self recompute
+    after [expr 6*60*60*1000] [list $self recompute_thread]
+}
+
+# Set-up "Alarmer::fire" to run on next minute boundary
+method Alarmer fire_thread {} {
+    set msec [expr "(60-[ical_time second [ical_time now]])*1000"]
+    if {$msec <= 0} {set msec 1}
+    after $msec [list $self fire]
+}
+
+# Clears shutup list every hour
+method Alarmer clear_shutup_thread {} {
+    # Remove all shutup flags
+    foreach n [array names slot shutup:*] {
+	unset slot($n)
+    }
+
+    # Restore shutup flags set within the last hour so that we do
+    # not lose recent shutups
+    foreach uid $slot(shutlist) {
+	set slot(shutup:$uid) 1
+    }
+
+    # Forget the shutup list for this past hour.  When this thread
+    # runs again in an hour, it will clear out any shutup flags
+    # that we just set.
+    set slot(shutlist) ""
+
+    # Reschedule
+    after [expr 60 * 60 * 1000] [list $self clear_shutup_thread]
 }
 
 # effects - Recompute alarm list
@@ -33,7 +67,7 @@ method Alarmer recompute {} {
     set slot(alarms) [cal option DefaultAlarms]
     set slot(pending) ""
 
-    set now [time now]
+    set now [ical_time now]
     set today [date today]
     set midnight [$self midnight]
 
@@ -54,19 +88,16 @@ method Alarmer recompute {} {
 method Alarmer add {item} {
     if ![$item is appt] return
 
-    set now [time now]
+    set now [ical_time now]
     set midnight [$self midnight]
 
-    set added 0
     set today [date today]
     if [$item contains $today] {
 	$self appt $item [expr $midnight+[$item starttime]*60] $now
-	set added 1
     }
 
     if [$item contains [expr $today+1]] {
 	$self appt $item [expr $midnight+(24*60*60)+[$item starttime]*60] $now
-	set added 1
     }
 }
 
@@ -78,8 +109,8 @@ method Alarmer change {item} {
 
 # effects - Get time today started
 method Alarmer midnight {} {
-    set now [time now]
-    set split [time split $now]
+    set now [ical_time now]
+    set split [ical_time split $now]
     set offset [expr "([lindex $split 0]*60*60 +\
 		       [lindex $split 1]*60 +\
 		       [lindex $split 2])"]
@@ -102,47 +133,66 @@ method Alarmer appt {appt time now} {
 	    continue
 	}
 
-	lappend slot(pending) [list $appt $t $time]
+	lappend slot(pending) $appt $t $time
     }
 }
 
 # effects - Remove pending entries for specified item
 method Alarmer remove {item} {
     set pending ""
-    foreach x $slot(pending) {
-	if {[lindex $x 0] != $item} {
-	    lappend pending $x
+    foreach {x fire start} $slot(pending) {
+	if {$x != $item} {
+	    lappend pending $x $fire $start
 	}
     }
     set slot(pending) $pending
 }
 
+# effects - Shutup on alarms within the next hour for specified item
+method Alarmer shutup {item} {
+    # We set the shutup flag for this item, and also add it
+    # to "shutlist" so that the shutup clearing thread will
+    # leave this shutup flag alone the next time it runs
+    set uid [list [$item uid] [$item text] [$item starttime]]
+    set slot(shutup:$uid) 1
+    lappend slot(shutlist) $uid
+}
+
 # effects - Fire pending alarms
 method Alarmer fire {} {
     set pending ""
 
     # Fire alarms a little early (helps bypass round-off errors)
-    set now [expr [time now]+5]
+    set now [expr [ical_time now]+5]
 
-    foreach x $slot(pending) {
-	if {[lindex $x 1] > $now} {
+    foreach {item fire start} $slot(pending) {
+	if {$fire > $now} {
 	    # Not time to fire yet
-	    lappend pending $x
+	    lappend pending $item $fire $start
+	    continue
+	}
+
+	# If alarms are shut off for this item, then ignore
+	set uid [list [$item uid] [$item text] [$item starttime]]
+	if [info exists slot(shutup:$uid)] {
 	    continue
 	}
 
 	# Close active notices
-	trigger fire kill_alarm [lindex $x 0]
+	trigger fire kill_alarm $item
 
 	# Create alarm
-	AlarmNotice [lindex $x 0] [lindex $x 2]
-	run-hook alarm-fire [lindex $x 0]
+	AlarmNotice $item $start
+	run-hook alarm-fire $item
     }
 
     set slot(pending) $pending
 
     # Update clocks in notices
     trigger fire update_alarms
+
+    # Set-up for next minute boundary
+    $self fire_thread
 }
 
 #### Alarm notices ####
@@ -152,20 +202,16 @@ class AlarmNotice {item starttime} {
     set slot(starttime) $starttime
 
     toplevel .$self -class Reminder
+    set_geometry {} .$self [option get .$self geometry Geometry]
 
     wm title .$self Reminder
     wm iconname .$self Reminder
-    wm protocol .$self WM_DELETE_WINDOW [list class_kill $self]
-
-    set g [option get .$self geometry Geometry]
-    if {$g != ""} {
-	catch {wm geometry .$self $g}
-    }
+    wm protocol .$self WM_DELETE_WINDOW [list AN_check_kill $self]
 
     # Buttons
     make_buttons .$self.bot 0\
 	[list\
-	     [list {Snooze}		[list class_kill $self]]\
+	     [list {Snooze}		[list AN_check_kill $self]]\
 	     [list {No More Alarms}	[list AN_shutup $self]]]
 				   
     # Display
@@ -195,8 +241,8 @@ class AlarmNotice {item starttime} {
     pack .$self.bot -side bottom -fill x
 
     # Key bindings
-    bind .$self <Control-c> [list class_kill $self]
-    bind .$self <Return> [list class_kill $self]
+    bind .$self <Control-c> [list AN_check_kill $self]
+    bind .$self <Return> [list AN_check_kill $self]
 
     # Triggers
     trigger on delete		[list AN_item_kill $self]
@@ -223,24 +269,24 @@ method AlarmNotice destructor {} {
 # effects  Kill the specified alarm notice and also remove any
 #	   pending alarms for the same item.
 proc AN_shutup {object} {
-    alarmer remove [$object item]
-    class_kill $object
+    catch {alarmer shutup [$object item]}
+    catch {class_kill $object}
 }
 
 # effects  Kill the specified alarm notice if it belongs to the
 #	   specified item.
 proc AN_item_kill {object item} {
-    if ![string compare [$object item] $item] {
-	class_kill $object
+    catch {
+	if ![string compare [$object item] $item] {
+	    class_kill $object
+	}
     }
 }
 
 # effects  Kill the specified alarm notice if its corresponding item
 #	   no longer exists.
 proc AN_check_kill {object} {
-    if ![string compare [info commands [$object item]] {}] {
-	class_kill $object
-    }
+    catch {class_kill $object}
 }
 
 method AlarmNotice item {} {
@@ -248,7 +294,7 @@ method AlarmNotice item {} {
 }
 
 method AlarmNotice countdown {} {
-    set now [time now]
+    set now [ical_time now]
     if {$now > $slot(starttime)} {
 	catch {.$self.foot configure -text "Late for Appointment"}
 
@@ -266,34 +312,13 @@ proc start_alarmer {} {
     Alarmer-with-name alarmer
 }
 
-#### Thread that runs at minute boundaries ####
-
-proc thread_minute {cmd} {
-    set id [_thread_alloc]
-    _thread_minute_schedule $id $cmd
-    return $id
-}
-
-proc _thread_minute_schedule {id cmd} {
-    # Schedule next firing (at minute boundary)
-    set msec [expr "(60-[time second [time now]])*1000"]
-    if {$msec <= 0} {set msec 1}
-    after $msec [list _thread_minute_fire $id $cmd]
-}
-
-proc _thread_minute_fire {id cmd} {
-    global _thread
-    if ![info exists _thread(run:$id)] {return}
-
-    eval $cmd
-    _thread_minute_schedule $id $cmd
-}
-
 #### Test list ####
 
-# - Alarms go off
-# - Alarm notices count down properly
-# - Item changes/deletions kill alarms
-# - Item deletions by other calendar instances kills alarms
-# - Alarm notices aren't duplicated
-#*- Shutup disables alarms
+# + Alarms go off
+# + Alarm notices aren't duplicated
+# + Alarm notices count down properly
+# + Item changes/deletions kill alarms
+# + Item deletions by other calendar instances kills alarms
+# + Shutup disables alarms
+# + Shutup disables alarms even if calendar is modified by someone else
+# + Shutup is overridden by changes
