@@ -11,6 +11,7 @@ class Lexer;
 class OptionMap;
 class pointerArray;
 class UidSet;
+class charArray;
 
 class Calendar {
   public:
@@ -74,10 +75,15 @@ class Calendar {
     int readonly;			// Readonly calendar?
     UidSet*	  hidden;		// Hidden items from other calendars
     OptionMap*	  options;		// Calendar options
+    int		  major, minor;		// Calendar file version numbers
 
     void clear();
     // modifies	this
     // effects	Restores calendar to pristine state.
+
+    void WriteV1(charArray*) const;
+    void WriteLatestVersion(charArray*) const;
+    void WriteCommon(charArray*) const;
 };
 
 inline int Calendar::ReadOnly() const {
