@@ -45,10 +45,6 @@ inline bool OptionMap::fetch(char const* key, char const*& value) const {
     return (rep->fetch(key, value));
 }
 
-inline void OptionMap::remove(char const* key) {
-    rep->remove(key);
-}
-
 inline OptionMap_Bindings OptionMap::bindings() const {
     OptionMap_Bindings b = rep;
     return b;
