@@ -26,15 +26,31 @@ class Ical_Doc {doc} {
     bind .$self <BackSpace> [list $slot(display) prev_page]
 
     set t $slot(text)
+
+    # Set-up indentation
+    set m [winfo pixels $t 20p]
+    foreach i {1 2 3 4} {
+	set i1 [expr ($i-1) * $m]
+	set i2 [expr $i * $m]
+	$t tag configure indent${i}h  -lmargin1 $i1 -lmargin2 $i2
+	$t tag configure indent${i}   -lmargin1 $i2 -lmargin2 $i2
+    }
+
+    # Headers
     $t tag configure header1	-font [pref largeHeadingFont]
     $t tag configure header2	-font [pref smallHeadingFont] -underline 1
     $t tag configure header3	-font [pref smallHeadingFont]
+
+    # Fonts
     $t tag configure norm	-font [pref normFont]
     $t tag configure bold	-font [pref boldFont]
     $t tag configure italic	-font [pref italFont]
-    $t tag configure fixed	-font [pref normFont]
-    $t tag configure boldfixed	-font [pref boldFont]
-    $t tag configure pre	-font [pref normFont]
+    $t tag configure fixed	-font [option get . normFixedFont Font]
+    $t tag configure boldfixed	-font [option get . boldFixedFont Font]
+    $t tag configure pre	-font [option get . normFixedFont Font]\
+				-wrap none
+
+    # References
     $t tag configure ref	-font [pref italFont]\
 				-foreground [pref interestColor]\
 				-underline 1
@@ -84,27 +100,25 @@ method Ical_Doc make_toc {} {
     set headers {}
 
     foreach level {1 2} {
-	set ranges [$t tag ranges header$level]
-	while {[llength $ranges] > 1} {
-	    set text [$t get [lindex $ranges 0] [lindex $ranges 1]]
+	foreach {s f} [join [$t tag ranges header$level]] {
+	    set text [$t get $s $f]
 	    set pad $level
 	    while {$pad > 1} {
 		set text "        $text"
 		incr pad -1
 	    }
-	    lappend headers [list [lindex $ranges 0] $text]
-	    set ranges [lrange $ranges 2 end]
+	    lappend headers [list $s $text]
 	}
     }
 
     # Sort headers by position in text
-    set headers [lsort -command "$self sort_by_pos" $headers]
+    set headers [lsort -command [list $self sort_by_pos] $headers]
 
     # Insert into box
     set i 0
-    foreach h $headers {
-	$l insert end [lindex $h 1]
-	$t mark set header_$i [lindex $h 0]
+    foreach {index text} [join $headers] {
+	$l insert end $text
+	$t mark set header_$i $index
 	incr i
     }
 
