@@ -21,6 +21,7 @@ proc ical_tk_script {} {
     # Load calendar
     calendar cal $ical(calendar)
     trigger fire reconfig
+    trigger fire keybind
 
     if {$popup} {
 	# Create item listing
@@ -49,8 +50,24 @@ proc ical_tk_script {} {
     }
 }
 
+# effects Startup a thread that will do the right thing at each midnight.
+proc start_midnight_thread {} {
+    set now [ical_time now]
+    set split [ical_time split $now]
+    set offset [expr "([lindex $split 0]*60*60 +\
+		       [lindex $split 1]*60 +\
+		       [lindex $split 2])"]
+
+    # Find time remaining today and schedule an event after that much time.
+    # Note: the "after" command takes time in milliseconds.
+    after [expr (24*60*60 - $offset)*1000] {
+	trigger fire midnight
+	start_midnight_thread
+    }
+}
+
 # Handle background errors
-proc tkerror {message} {
+proc bgerror {message} {
     global ical errorInfo
 
     # XXX Fairly stupid handling of unknown color errors
