@@ -12,6 +12,7 @@ class Preference {} {
 
 method Preference load_common {} {
     option add *fontFamily		times		startupFile
+    option add *fixedFontFamily		courier		startupFile
     option add *Font			fixed		startupFile
     option add *fontSize		normal		startupFile
 
@@ -24,11 +25,9 @@ method Preference load_common {} {
 
     option add *Dayview*takeFocus	0		startupFile
 
-    option add *ApptList.Canvas.BorderWidth		1	startupFile
     option add *ApptList.Canvas.Relief			raised	startupFile
     option add *ApptList.Canvas.highlightThickness	0	startupFile
 
-    option add *NoteList.Canvas.BorderWidth		1	startupFile
     option add *NoteList.Canvas.Relief			raised	startupFile
     option add *NoteList.Canvas.highlightThickness	0	startupFile
 }
@@ -66,6 +65,9 @@ method Preference load_color {} {
     option add *itemOverflowColor	$over		startupFile
     option add *itemOverflowStipple	{}		startupFile
     option add *Canvas*selectBackground	$csbg		startupFile
+
+    option add *ApptList.Canvas.BorderWidth	1	startupFile
+    option add *NoteList.Canvas.BorderWidth	1	startupFile
 }
 
 method Preference load_mono {} {
@@ -85,15 +87,16 @@ method Preference load_mono {} {
     option add *itemSelectWidth		4		startupFile
     option add *itemOverflowColor	black		startupFile
     option add *itemOverflowStipple	gray50		startupFile
+
+    option add *ApptList.Canvas.BorderWidth	2	startupFile
+    option add *NoteList.Canvas.BorderWidth	2	startupFile
 }
 
 method Preference reload {} {
     option clear
     tcllib_load_options
-    $self load_common
 
     # Load default options
-    global ical
     $self load_common
     if [string match *color* [winfo screenvisual .]] {
 	$self load_color
@@ -120,19 +123,28 @@ method Preference reload {} {
     }
 
     # Handle command line preferences
+    global ical
     foreach pref $ical(prefs) {eval $pref}
 }
 
 # Fix fonts in option database
 method Preference fixfonts {} {
-    set slot(fontfamilies) [list\
-			    [option get . fontFamily String]\
-			    times\
-			    charter\
-			    {new century schoolbook}\
-			    courier\
-			    helvetica\
-			   ]
+    set ff [option get . fontFamily String]
+    set slot(norm_fontfamilies) [concat [list $ff] {
+	times
+	charter
+	{new century schoolbook}
+	courier
+	helvetica
+    }]
+
+    set ff [option get . fixedFontFamily String]
+    set slot(fixed_fontfamilies) [concat [list $ff] {
+	courier
+	fixed
+	terminal
+	lucidatypewriter
+    }]
 
     switch -exact -- [option get . fontSize String] {
 	small {
@@ -149,17 +161,21 @@ method Preference fixfonts {} {
 	}
     }
 
-    # Find fonts
-    set canvas [canvas .preffonts]
-    set norm120		[$self findfont $canvas medium r $size1]
-    set norm140		[$self findfont $canvas medium r $size2]
-    set ital140		[$self findfont $canvas medium i $size2]
-    set bold140		[$self findfont $canvas bold   r $size2]
-    set blit140		[$self findfont $canvas bold   i $size2]
-    set norm180		[$self findfont $canvas medium r $size3]
-    set bold180		[$self findfont $canvas bold   r $size3]
-    set norm240		[$self findfont $canvas medium r $size4]
-    destroy $canvas
+    # Normal fonts
+    set ff norm_fontfamilies
+    set norm120		[$self findfont medium r $size1 $ff]
+    set norm140		[$self findfont medium r $size2 $ff]
+    set ital140		[$self findfont medium i $size2 $ff]
+    set bold140		[$self findfont bold   r $size2 $ff]
+    set blit140		[$self findfont bold   i $size2 $ff]
+    set norm180		[$self findfont medium r $size3 $ff]
+    set bold180		[$self findfont bold   r $size3 $ff]
+    set norm240		[$self findfont medium r $size4 $ff]
+
+    # Fixed fonts
+    set ff fixed_fontfamilies
+    set norm140fixed	[$self findfont medium r $size2 $ff]
+    set bold140fixed	[$self findfont medium r $size2 $ff]
 
     # Set option database
     option add *weekdayFont		$norm140 startupFile
@@ -175,6 +191,9 @@ method Preference fixfonts {} {
     option add *smallHeadingFont	$bold140 startupFile
     option add *largeHeadingFont	$bold180 startupFile
 
+    option add *normFixedFont		$norm140fixed startupFile
+    option add *boldFixedFont		$bold140fixed startupFile
+
     # General preferences
     option add *Dialog*font		$norm180 startupFile
     option add *Dialog*Message*font	$norm140 startupFile
@@ -200,18 +219,19 @@ method Preference fixfonts {} {
 }
 
 # effects - Find font matching given specification
-method Preference findfont {canvas weight style size} {
-    # Search for this size in families
-    foreach family $slot(fontfamilies) {
-	set f "-*-$family-$weight-$style-normal-*-*-$size-*"
-	if ![catch {$canvas create text 0 0 -text XXX -font $f}] {
-	    $canvas delete all
-	    return $f
+method Preference findfont {weight style size ff} {
+    # Try a whole bunch of sizes
+    foreach sdelta {0 10 -10 20 -20 30 -30 40 -40} {
+	# Search for this size in families
+	set s [expr $size + $sdelta]
+	foreach family $slot($ff) {
+	    set f "-*-$family-$weight-$style-normal-*-*-$s-*"
+	    if [font_exists $f] {
+		return $f
+	    }
 	}
     }
 
-    # XXX - More searching?
-
     # Return default font
     return fixed
 }
@@ -230,36 +250,45 @@ method Preference init {} {
 
 # Methods for obtaining various preferences.
 
+# Return the specified font if it exists, else return fixed.
+proc pref_cf {f} {
+    if ![font_exists $f] {
+	return fixed
+    } else {
+	return $f
+    }
+}
+
 method Preference weekdayFont {} {
-    return [option get . weekdayFont Font]
+    return [pref_cf [option get . weekdayFont Font]]
 }
 
 method Preference weekendFont {} {
-    return [option get . weekendFont Font]
+    return [pref_cf [option get . weekendFont Font]]
 }
 
 method Preference interestFont {} {
-    return [option get . interestFont Font]
+    return [pref_cf [option get . interestFont Font]]
 }
 
 method Preference weekendInterestFont {} {
-    return [option get . weekendInterestFont Font]
+    return [pref_cf [option get . weekendInterestFont Font]]
 }
 
 method Preference itemFont {} {
-    return [option get . itemFont Font]
+    return [pref_cf [option get . itemFont Font]]
 }
 
-method Preference normFont {} {return [option get . normFont Font]}
-method Preference boldFont {} {return [option get . boldFont Font]}
-method Preference italFont {} {return [option get . italFont Font]}
+method Preference normFont {} {return [pref_cf [option get . normFont Font]]}
+method Preference boldFont {} {return [pref_cf [option get . boldFont Font]]}
+method Preference italFont {} {return [pref_cf [option get . italFont Font]]}
 
 method Preference smallHeadingFont {} {
-    return [option get . smallHeadingFont Font]
+    return [pref_cf [option get . smallHeadingFont Font]]
 }
 
 method Preference largeHeadingFont {} {
-    return [option get . largeHeadingFont Font]
+    return [pref_cf [option get . largeHeadingFont Font]]
 }
 
 method Preference weekdayColor {} {
