@@ -82,11 +82,11 @@ proc bug_interact {mailer email message} {
     $f.warn configure -text [format [join {
 	"An internal error has occurred.  You can send a bug report to"
 	"the author (%s) by clicking on the \"Mail Report\" button"
-	"at the bottom of this dialog.  The text displayed below"
-	"will be the only information sent to the author.  If you do not"
-	"want to send a bug report, click on the \"Dismiss\" button."
-	"If you want to send extra comments to the author, type them into"
-	"the area below."
+	"at the bottom of this dialog.  The text displayed in the text"
+	"areas below will be the only information sent to the author."
+        "If you do not want to send a bug report, click on the"
+	"\"Dismiss\" button.  If you want to send extra comments to"
+        "the author, type them into the area below."
     }] $email]
 
     set mtext [$bug(message) text]
