@@ -6,7 +6,6 @@
 // creation time.
 
 #include <assert.h>
-#include "config.h"
 
 typedef unsigned long int bitvword;
 
@@ -51,6 +50,11 @@ class BitVec {
     // modifies	"this"
     // effects	Removes all bits from vector.
 
+    void invert();
+    // modifies	"this"
+    // effects	foreach i in [0..length()-1]
+    //		    post(value(i)) == !pre(value(i))
+
     int first_set() const;
     // effects	Return number of smallest bit contained in bit vector.
     //		If bit vector is empty, then returns "length()".
@@ -73,12 +77,15 @@ class BitVec {
     bitvword word_mask(int n) const;	// Mask for extracting bit "n"
 };
 
-#if (SIZEOF_LONG == 8)
-#define BITVEC_WSIZE  64
-#define BITVEC_WSHIFT 6
-#else
 #define BITVEC_WSIZE  32
 #define BITVEC_WSHIFT 5
+
+// If longs are actually bigger, exploit that fact.
+#ifdef __alpha
+#undef BITVEC_WSIZE
+#undef BITVEC_WSHIFT
+#define BITVEC_WSIZE  64
+#define BITVEC_WSHIFT 6
 #endif
 
 inline int BitVec::word_size() const	   { return BITVEC_WSIZE; }
